All of lore.kernel.org
 help / color / mirror / Atom feed
* [PATCH 1/5] fat: add i_disksize to represent uninitialized size
@ 2013-10-09 14:40 Namjae Jeon
  2013-10-27 13:38 ` OGAWA Hirofumi
  0 siblings, 1 reply; 5+ messages in thread
From: Namjae Jeon @ 2013-10-09 14:40 UTC (permalink / raw)
  To: hirofumi, akpm
  Cc: linux-fsdevel, linux-kernel, a.sahrawat, Namjae Jeon, Namjae Jeon

From: Namjae Jeon <namjae.jeon@samsung.com>

Add i_disksize to represent uninitialized allocated size.
And mmu_private represent initialized allocated size.

Signed-off-by: Namjae Jeon <namjae.jeon@samsung.com>
Signed-off-by: Amit Sahrawat <a.sahrawat@samsung.com>
---
 fs/fat/cache.c |    4 ++--
 fs/fat/fat.h   |    3 ++-
 fs/fat/file.c  |    4 +++-
 fs/fat/inode.c |    4 ++++
 4 files changed, 11 insertions(+), 4 deletions(-)

diff --git a/fs/fat/fat.h b/fs/fat/fat.h
index 4241e6f..1716199 100644
--- a/fs/fat/fat.h
+++ b/fs/fat/fat.h
@@ -117,7 +117,8 @@ struct msdos_inode_info {
 	unsigned int cache_valid_id;
 
 	/* NOTE: mmu_private is 64bits, so must hold ->i_mutex to access */
-	loff_t mmu_private;	/* physically allocated size */
+	loff_t mmu_private;	/* physically allocated size (uninitialized) */
+	loff_t i_disksize;	/* physically allocated size (initialized) */
 
 	int i_start;		/* first cluster or 0 */
 	int i_logstart;		/* logical first cluster */
diff --git a/fs/fat/file.c b/fs/fat/file.c
index 33711ff..7301adb 100644
--- a/fs/fat/file.c
+++ b/fs/fat/file.c
@@ -300,8 +300,10 @@ void fat_truncate_blocks(struct inode *inode, loff_t offset)
 	 * This protects against truncating a file bigger than it was then
 	 * trying to write into the hole.
 	 */
-	if (MSDOS_I(inode)->mmu_private > offset)
+	if (MSDOS_I(inode)->mmu_private > offset) {
 		MSDOS_I(inode)->mmu_private = offset;
+		MSDOS_I(inode)->i_disksize = offset;
+	}
 
 	nr_clusters = (offset + (cluster_size - 1)) >> sbi->cluster_bits;
 
diff --git a/fs/fat/inode.c b/fs/fat/inode.c
index 3134d1e..578a5db 100644
--- a/fs/fat/inode.c
+++ b/fs/fat/inode.c
@@ -93,6 +93,7 @@ static inline int __fat_get_block(struct inode *inode, sector_t iblock,
 
 	*max_blocks = min(mapped_blocks, *max_blocks);
 	MSDOS_I(inode)->mmu_private += *max_blocks << sb->s_blocksize_bits;
+	MSDOS_I(inode)->i_disksize = MSDOS_I(inode)->mmu_private;
 
 	err = fat_bmap(inode, iblock, &phys, &mapped_blocks, create);
 	if (err)
@@ -406,6 +407,7 @@ int fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de)
 		if (error < 0)
 			return error;
 		MSDOS_I(inode)->mmu_private = inode->i_size;
+		MSDOS_I(inode)->i_disksize = inode->i_size;
 
 		set_nlink(inode, fat_subdirs(inode));
 	} else { /* not a directory */
@@ -421,6 +423,7 @@ int fat_fill_inode(struct inode *inode, struct msdos_dir_entry *de)
 		inode->i_fop = &fat_file_operations;
 		inode->i_mapping->a_ops = &fat_aops;
 		MSDOS_I(inode)->mmu_private = inode->i_size;
+		MSDOS_I(inode)->i_disksize = inode->i_size;
 	}
 	if (de->attr & ATTR_SYS) {
 		if (sbi->options.sys_immutable)
@@ -1218,6 +1221,7 @@ static int fat_read_root(struct inode *inode)
 			   & ~((loff_t)sbi->cluster_size - 1)) >> 9;
 	MSDOS_I(inode)->i_logstart = 0;
 	MSDOS_I(inode)->mmu_private = inode->i_size;
+	MSDOS_I(inode)->i_disksize = inode->i_size;
 
 	fat_save_attrs(inode, ATTR_DIR);
 	inode->i_mtime.tv_sec = inode->i_atime.tv_sec = inode->i_ctime.tv_sec = 0;
diff --git a/fs/fat/cache.c b/fs/fat/cache.c
index 91ad9e1..37572c2 100644
--- a/fs/fat/cache.c
+++ b/fs/fat/cache.c
@@ -329,10 +329,10 @@ int fat_bmap(struct inode *inode, sector_t sector, sector_t *phys,
 			return 0;
 
 		/*
 		 * ->mmu_private can access on only allocation path.
 		 * (caller must hold ->i_mutex)
 		 */
-		last_block = (MSDOS_I(inode)->mmu_private + (blocksize - 1))
+		last_block = (MSDOS_I(inode)->i_disksize + (blocksize - 1))
 			>> blocksize_bits;
 		if (sector >= last_block)
 			return 0;
@@ -329,10 +329,10 @@ int fat_bmap(struct inode *inode, sector_t sector, sector_t *phys,
 
 		/*
-		 * ->mmu_private can access on only allocation path.
+		 * ->i_disksize can access on only allocation path.
 		 * (caller must hold ->i_mutex)
 		 */
-- 
1.7.9.5


^ permalink raw reply related	[flat|nested] 5+ messages in thread

* Re: [PATCH 1/5] fat: add i_disksize to represent uninitialized size
  2013-10-09 14:40 [PATCH 1/5] fat: add i_disksize to represent uninitialized size Namjae Jeon
@ 2013-10-27 13:38 ` OGAWA Hirofumi
  2013-10-29 23:01   ` Namjae Jeon
  0 siblings, 1 reply; 5+ messages in thread
From: OGAWA Hirofumi @ 2013-10-27 13:38 UTC (permalink / raw)
  To: Namjae Jeon; +Cc: akpm, linux-fsdevel, linux-kernel, a.sahrawat, Namjae Jeon

Namjae Jeon <linkinjeon@gmail.com> writes:

Code seems to be much became simpler.

> diff --git a/fs/fat/cache.c b/fs/fat/cache.c
> index 91ad9e1..37572c2 100644
> --- a/fs/fat/cache.c
> +++ b/fs/fat/cache.c
> @@ -329,10 +329,10 @@ int fat_bmap(struct inode *inode, sector_t sector, sector_t *phys,
>  			return 0;
>  
>  		/*
> -		 * ->mmu_private can access on only allocation path.
> +		 * ->i_disksize can access on only allocation path.
>  		 * (caller must hold ->i_mutex)
>  		 */
> -		last_block = (MSDOS_I(inode)->mmu_private + (blocksize - 1))
> +		last_block = (MSDOS_I(inode)->i_disksize + (blocksize - 1))
>  			>> blocksize_bits;
>  		if (sector >= last_block)
>  			return 0;

Hm, bmap() ioctl returns between ->mmu_private and i_disksize? I'm not
checking other FSes what does...
-- 
OGAWA Hirofumi <hirofumi@mail.parknet.co.jp>

^ permalink raw reply	[flat|nested] 5+ messages in thread

* Re: [PATCH 1/5] fat: add i_disksize to represent uninitialized size
  2013-10-27 13:38 ` OGAWA Hirofumi
@ 2013-10-29 23:01   ` Namjae Jeon
  2013-10-30  1:23     ` OGAWA Hirofumi
  0 siblings, 1 reply; 5+ messages in thread
From: Namjae Jeon @ 2013-10-29 23:01 UTC (permalink / raw)
  To: OGAWA Hirofumi; +Cc: akpm, linux-fsdevel, linux-kernel, a.sahrawat, Namjae Jeon

2013/10/27, OGAWA Hirofumi <hirofumi@mail.parknet.co.jp>:
> Namjae Jeon <linkinjeon@gmail.com> writes:
Hi OGAWA.
>
> Code seems to be much became simpler.
Thanks :)
>
>> diff --git a/fs/fat/cache.c b/fs/fat/cache.c
>> index 91ad9e1..37572c2 100644
>> --- a/fs/fat/cache.c
>> +++ b/fs/fat/cache.c
>> @@ -329,10 +329,10 @@ int fat_bmap(struct inode *inode, sector_t sector,
>> sector_t *phys,
>>  			return 0;
>>
>>  		/*
>> -		 * ->mmu_private can access on only allocation path.
>> +		 * ->i_disksize can access on only allocation path.
>>  		 * (caller must hold ->i_mutex)
>>  		 */
>> -		last_block = (MSDOS_I(inode)->mmu_private + (blocksize - 1))
>> +		last_block = (MSDOS_I(inode)->i_disksize + (blocksize - 1))
>>  			>> blocksize_bits;
>>  		if (sector >= last_block)
>>  			return 0;
>
> Hm, bmap() ioctl returns between ->mmu_private and i_disksize? I'm not
> checking other FSes what does...
I added this code after checking such behaviour from ext4.

Thanks.
> --
> OGAWA Hirofumi <hirofumi@mail.parknet.co.jp>
>

^ permalink raw reply	[flat|nested] 5+ messages in thread

* Re: [PATCH 1/5] fat: add i_disksize to represent uninitialized size
  2013-10-29 23:01   ` Namjae Jeon
@ 2013-10-30  1:23     ` OGAWA Hirofumi
  2013-10-30  4:34       ` Namjae Jeon
  0 siblings, 1 reply; 5+ messages in thread
From: OGAWA Hirofumi @ 2013-10-30  1:23 UTC (permalink / raw)
  To: Namjae Jeon; +Cc: akpm, linux-fsdevel, linux-kernel, a.sahrawat, Namjae Jeon

Namjae Jeon <linkinjeon@gmail.com> writes:

>>> diff --git a/fs/fat/cache.c b/fs/fat/cache.c
>>> index 91ad9e1..37572c2 100644
>>> --- a/fs/fat/cache.c
>>> +++ b/fs/fat/cache.c
>>> @@ -329,10 +329,10 @@ int fat_bmap(struct inode *inode, sector_t sector,
>>> sector_t *phys,
>>>  			return 0;
>>>
>>>  		/*
>>> -		 * ->mmu_private can access on only allocation path.
>>> +		 * ->i_disksize can access on only allocation path.
>>>  		 * (caller must hold ->i_mutex)
>>>  		 */
>>> -		last_block = (MSDOS_I(inode)->mmu_private + (blocksize - 1))
>>> +		last_block = (MSDOS_I(inode)->i_disksize + (blocksize - 1))
>>>  			>> blocksize_bits;
>>>  		if (sector >= last_block)
>>>  			return 0;
>>
>> Hm, bmap() ioctl returns between ->mmu_private and i_disksize? I'm not
>> checking other FSes what does...
> I added this code after checking such behaviour from ext4.

OK. I will check with it. BTW, comment should say the both
(i.e. ->mmu_private and ->i_disksize must under ->i_mutex).
-- 
OGAWA Hirofumi <hirofumi@mail.parknet.co.jp>

^ permalink raw reply	[flat|nested] 5+ messages in thread

* Re: [PATCH 1/5] fat: add i_disksize to represent uninitialized size
  2013-10-30  1:23     ` OGAWA Hirofumi
@ 2013-10-30  4:34       ` Namjae Jeon
  0 siblings, 0 replies; 5+ messages in thread
From: Namjae Jeon @ 2013-10-30  4:34 UTC (permalink / raw)
  To: OGAWA Hirofumi; +Cc: akpm, linux-fsdevel, linux-kernel, a.sahrawat, Namjae Jeon

2013/10/30, OGAWA Hirofumi <hirofumi@mail.parknet.co.jp>:
> Namjae Jeon <linkinjeon@gmail.com> writes:
>
>>>> diff --git a/fs/fat/cache.c b/fs/fat/cache.c
>>>> index 91ad9e1..37572c2 100644
>>>> --- a/fs/fat/cache.c
>>>> +++ b/fs/fat/cache.c
>>>> @@ -329,10 +329,10 @@ int fat_bmap(struct inode *inode, sector_t
>>>> sector,
>>>> sector_t *phys,
>>>>  			return 0;
>>>>
>>>>  		/*
>>>> -		 * ->mmu_private can access on only allocation path.
>>>> +		 * ->i_disksize can access on only allocation path.
>>>>  		 * (caller must hold ->i_mutex)
>>>>  		 */
>>>> -		last_block = (MSDOS_I(inode)->mmu_private + (blocksize - 1))
>>>> +		last_block = (MSDOS_I(inode)->i_disksize + (blocksize - 1))
>>>>  			>> blocksize_bits;
>>>>  		if (sector >= last_block)
>>>>  			return 0;
>>>
>>> Hm, bmap() ioctl returns between ->mmu_private and i_disksize? I'm not
>>> checking other FSes what does...
>> I added this code after checking such behaviour from ext4.
>
> OK. I will check with it. BTW, comment should say the both
> (i.e. ->mmu_private and ->i_disksize must under ->i_mutex).
Okay, I will update.
Thanks!
> --
> OGAWA Hirofumi <hirofumi@mail.parknet.co.jp>
>

^ permalink raw reply	[flat|nested] 5+ messages in thread

end of thread, other threads:[~2013-10-30  4:34 UTC | newest]

Thread overview: 5+ messages (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2013-10-09 14:40 [PATCH 1/5] fat: add i_disksize to represent uninitialized size Namjae Jeon
2013-10-27 13:38 ` OGAWA Hirofumi
2013-10-29 23:01   ` Namjae Jeon
2013-10-30  1:23     ` OGAWA Hirofumi
2013-10-30  4:34       ` Namjae Jeon

This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.